#######################################
## packages, all available from CRAN
library("psychotools", quietly=TRUE)
library("blavaan", quietly=TRUE)
library("semPlot", quietly=TRUE)
library("qgraph", quietly=TRUE)
library("semTools", quietly = TRUE)

#######################################
## random seed
set.seed(1050)

#######################################
## auxiliary code to create Figure 1
source("pathpd.R")

#######################################
## Theta matrix at bottom of p. 5
tmat <- matrix("",11,11)
diag(tmat) <- c(paste("\\theta_{", 1:9, 1:9, "}", sep=""),
                paste("\\theta_{", 10:11, ",", 10:11, "}", sep=""))
tmat[4,8] <- tmat[8,4] <- "\\theta_{48}"
tmat[5,7] <- tmat[7,5] <- "\\theta_{57}"
tmat[5,9] <- tmat[9,5] <- "\\theta_{59}"
tmat[6,10] <- tmat[10,6] <- "\\theta_{6,10}"
tmat[7,11] <- tmat[11,7] <- "\\theta_{7,11}"
tmat[9,11] <- tmat[11,9] <- "\\theta_{9,11}"
tmat <- apply(tmat, 1, paste, collapse=" & ")
tmat <- paste(tmat, "\\\\")
writeLines(tmat)

#######################################
## Figure 1
pdmodel <- ' 
         # latent variable definitions
           ind60 =~ x1 + x2 + x3
           dem60 =~ y1 + y2 + y3 + y4
           dem65 =~ y5 + y6 + y7 + y8
     
         # regressions
           dem60 ~ ind60
           dem65 ~ ind60 + dem60
     
         # residual correlations
           y1 ~~ y5
           y2 ~~ y4 + y6
           y3 ~~ y7
           y4 ~~ y8
           y6 ~~ y8
'

## to get path diagram:
mlfit <- sem(pdmodel, data=PoliticalDemocracy)
mlfit@ParTable$label[4:7] <- mlfit@ParTable$label[8:11] <- c(1, paste("lambda[",5:7,"]",sep=""))

layout(1:2)
pathpd(mlfit)

pdmodel2 <- ' 
          # latent variable definitions
            ind60 =~ x1 + x2 + x3
            dem60 =~ y1 + y2 + y3 + y4
            dem65 =~ y5 + y6 + y7 + y8

            D1 =~ y1 + y5
            D2 =~ y2 + y4
            D3 =~ y2 + y6
            D4 =~ y3 + y7
            D5 =~ y4 + y8
            D6 =~ y6 + y8
     
          # regressions
            dem60 ~ ind60
            dem65 ~ ind60 + dem60
'

## to get path diagram:
mlfit2 <- lavaan(pdmodel2, data=PoliticalDemocracy, auto.var=TRUE)
mlfit2@ParTable$label[4:7] <- mlfit2@ParTable$label[8:11] <- c(1, paste("lambda[",5:7,"]",sep=""))
pathpd2(mlfit2)

## ----dp, echo=TRUE-------------------------------------------------------
dpriors()

#######################################
## Section 4.1, political democracy example
## ----pdcode, echo=TRUE, eval=FALSE---------------------------------------
model <- '
       # latent variable definitions
         ind60 =~ x1 + x2 + x3
         dem60 =~ y1 + a*y2 + b*y3 + c*y4
         dem65 =~ y5 + a*y6 + b*y7 + c*y8

       # regressions
         dem60 ~ ind60
         dem65 ~ ind60 + dem60

       # residual correlations
         y1 ~~ y5
         y2 ~~ y4 + y6
         y3 ~~ y7
         y4 ~~ y8
         y6 ~~ y8
'

## ----pdmod2, echo=TRUE, eval=FALSE---------------------------------------
fit <- bsem(model, data = PoliticalDemocracy,
            dp = dpriors(nu = "dnorm(5,1e-2)", itheta = "dlnorm(1,.1)[sd]",
                         ipsi = "dlnorm(1,.1)[sd]", rho = "dbeta(3,3)"),
            jagcontrol = list(method = "rjparallel"), seed = 999:1001)

## ----tspl,echo=FALSE,message=FALSE,fig.align="center",fig.width=7,fig.height=7,out.width="4.5in",out.height="4.5in"----
plot(fit, 1:4, plot.type = "trace", new.windows = FALSE)

## ----speedtest, echo=FALSE, eval=FALSE-----------------------------------
system.time(fit1 <- bsem(model, data=PoliticalDemocracy, dp=dpriors(nu="dnorm(5,1e-2)"), jagcontrol=list(method="rjparallel"), convergence="manual", test="none", seed = 1004:1006))

system.time(fit2 <- bsem(model, data=PoliticalDemocracy, dp=dpriors(nu="dnorm(5,1e-2)"), jagcontrol=list(method="rjparallel"), convergence="manual", cp="fa", test="none", seed = 1004:1006))

cbind(blavInspect(fit1,'neff'), blavInspect(fit2,'neff'))
summary(blavInspect(fit1,'neff')/blavInspect(fit2,'neff'))

## NOTE: The ppp displayed below in summary() and fitMeasures() will vary
##       in the third decimal place because blavaan uses mclapply() for
##       this computation, and blavaan doesn't allow the mclapply() seeds
##       to be set.
## ----pdsumm, echo=TRUE, eval=TRUE----------------------------------------
summary(fit)

## ----pdfitmeas, echo=TRUE, eval=TRUE-------------------------------------
fitMeasures(fit)


#######################################
## Section 4.2, measurement invariance example
## ----mimod, echo=TRUE, eval=TRUE----------------------------------------
HS.model <- ' visual  =~ x1 + x2 + x3
              textual =~ x4 + x5 + x6
              speed   =~ x7 + x8 + x9 '
fit1 <- cfa(HS.model, data = HolzingerSwineford1939, group = "school")
fit2 <- cfa(HS.model, data = HolzingerSwineford1939, group = "school",
             group.equal = "loadings")
fit3 <- cfa(HS.model, data = HolzingerSwineford1939, group = "school",
             group.equal = c("loadings", "intercepts"))

## ----mifitfun, echo=TRUE, eval=TRUE-------------------------------------
bfit1 <- bcfa(parTable(fit1), data = HolzingerSwineford1939,
              group = "school", seed = 1007:1009)
bfit2 <- bcfa(parTable(fit2), data = HolzingerSwineford1939,
              group = "school", seed = 1010:1012)
bfit3 <- bcfa(parTable(fit3), data = HolzingerSwineford1939,
              group = "school", seed = 1013:1015)

## NOTE: Again, ppp will vary in third decimal due to mclapply()
## ----mifitmeas, echo=TRUE, eval=TRUE-------------------------------------
fitMeasures(bfit1)
fitMeasures(bfit2)


#######################################
## Section 4.3, informative prior example
## ----infpri, echo=TRUE, eval=FALSE---------------------------------------
library("psychotools")
data("StereotypeThreat")
st <- subset(StereotypeThreat, ethnicity == "majority")
model <- 'ability =~ abstract + verbal + numerical'
bfit <- bcfa(model, data = st, convergence = "auto", seed = 1012:1014)

## ----infpri2, echo=TRUE, eval=FALSE--------------------------------------
model <- c(model,
           'abstract ~~ prior("dunif(0,9)[sd]") * abstract
            verbal ~~ prior("dunif(0,8)[sd]") * verbal
            numerical ~~ prior("dunif(0,7)[sd]") * numerical')

## ----infpri3, echo=TRUE, eval=FALSE--------------------------------------
model <- c(model,
           'ability ~~ prior("dunif(0,4.5)[sd]") * ability')

## ----infpri4, echo=TRUE, eval=FALSE--------------------------------------
model <- c(model,
           'abstract ~ prior("dnorm(9,.25)T(0,18)") * 1
            verbal ~ prior("dnorm(8,.25)T(0,16)") * 1
            numerical ~ prior("dnorm(7,.25)T(0,14)") * 1')

## ----infpri5, echo=TRUE, eval=FALSE--------------------------------------
bfitInform <- bcfa(model, data = st,
                   dp = dpriors(lambda = "dunif(0,3)"),
                   convergence = "auto", seed = 1015:1017)

#######################################
## Section 4.3, Table 1
## ----wicpri, results="asis"----------------------------------------------
plabs <- parTable(bfit)
plabs <- plabs$pxnames[plabs$free > 0]
plabs <- sapply(strsplit(plabs, ","), head, 1)
plabs <- gsub("\\[", "_", plabs)
plabs <- paste("$\\", plabs, "$", sep="")
wictab <- rbind(coef(bfit), sqrt(diag(vcov(bfit))),
                coef(bfitInform), sqrt(diag(vcov(bfitInform))))
wictab <- round(wictab,2)
wictab <- rbind(plabs, wictab)
rowlab1 <- c("Prior", "Default", "", "Inform.", "")
rowlab2 <- c("", "Est", "SD", "Est", "SD")
wictab <- cbind(rowlab1, rowlab2, wictab)
wictab <- paste(apply(wictab, 1, paste, collapse = " & "), "\\\\")
wictab[c(1,3,5)] <- paste(wictab[c(1,3,5)], "\\hline")
writeLines(wictab)


#######################################
## Section 4.4, extensions of jags syntax
## ----robfa1, eval=FALSE, echo=TRUE---------------------------------------
model <- 'ability =~ abstract + verbal + numerical'
bfit <- bcfa(model, data = st, jagfile = TRUE)

## ----robfarun, echo=TRUE, eval=FALSE-------------------------------------
load("lavExport/robustfa.rda")
fit <- run.jags("lavExport/robustfa.jag",
                monitor = c(jagtrans$monitors, "df"),
                data = jagtrans$data, inits = jagtrans$inits)
